@@ -1,5 +1,5 @@
 //-----------------------------------------------------------------------------
-//  Copyright (C) 2017-2020 Thomas S. Ullrich
+//  Copyright (C) 2017-2022 Thomas S. Ullrich
 //
 //  This file is part of "xyscan".
 //
@@ -8,8 +8,9 @@
 //  under the terms of the GNU General Public License.
 //
 //  Author: Thomas S. Ullrich
-//  Last update: March 5, 2020
+//  Last update: Jan 10, 2022
 //-----------------------------------------------------------------------------
+#include <QLocale>
 #include <QHeaderView>
 #include <QPixmap>
 #include <QDir>
@@ -177,7 +178,7 @@ double xyscanDataTable::dx_lower(int irow, int ierror) const
     if (ierror > mXErrorScanCount-1) return 0;
     int k = 2 + 2*ierror;
     if (!item(irow, k)) return 0;
-    return item(irow, k)->text().toDouble();
+    return value(irow, k);
 }
 
 double xyscanDataTable::dx_upper(int irow, int ierror) const
@@ -185,7 +186,7 @@ double xyscanDataTable::dx_upper(int irow, int ierror) const
     if (ierror > mXErrorScanCount-1) return 0;
     int k = 2 + 2*ierror;
     if (!item(irow, k+1)) return 0;
-    return item(irow, k+1)->text().toDouble();
+    return value(irow, k+1);
 }
 
 double xyscanDataTable::dy_lower(int irow, int ierror) const
@@ -193,7 +194,7 @@ double xyscanDataTable::dy_lower(int irow, int ierror) const
     if (ierror > mYErrorScanCount-1) return 0;
     int k = 2 + 2*mXErrorScanCount + 2*ierror;
     if (!item(irow, k)) return 0;
-    return item(irow, k)->text().toDouble();
+    return value(irow, k);
 }
 
 double xyscanDataTable::dy_upper(int irow, int ierror) const
@@ -201,7 +202,16 @@ double xyscanDataTable::dy_upper(int irow, int ierror) const
     if (ierror > mYErrorScanCount-1) return 0;
     int k = 2 + 2*mXErrorScanCount + 2*ierror;
     if (!item(irow, k+1)) return 0;
-    return item(irow, k+1)->text().toDouble();
+    return value(irow, k+1);
+}
+
+void xyscanDataTable::resetAll()
+{
+    deleteAll();
+    resetColumns();
+    mXErrorScanCount = 0;
+    mYErrorScanCount = 0;
+    mUserComment = "";
 }
 
 void xyscanDataTable::resetColumns()
@@ -246,7 +256,7 @@ void xyscanDataTable::writeCsvFile(QFile& file, const QString&)
     for (int i=0; i < mYErrorScanCount; i++) {
         out << ", -dy, +dy";
     }
-    out << endl;
+    out << Qt::endl;
     int irow, icol;
     for (irow=0; irow < rowCount(); irow++) {
         for (icol=0; icol < columnCount(); icol++) {
@@ -256,7 +266,7 @@ void xyscanDataTable::writeCsvFile(QFile& file, const QString&)
                 out << "-";
             if (icol < columnCount()-1) out << ",";
         }
-        out << endl;
+        out << Qt::endl;
     }
     mDataSaved = true;
 }
@@ -366,45 +376,45 @@ void xyscanDataTable::writeRootMacro(QFile& file, const QString& src, bool logx,
     //
     //   Header
     //
-    out << "//" << endl;
-    out << tr("// ROOT macro: ") << QFileInfo(file).fileName().toLatin1() << tr(" (autogenerated by xyscan)") << endl;
-    out << tr("// xyscan Version ") << VERSION << endl;
+    out << "//" << Qt::endl;
+    out << tr("// ROOT macro: ") << QFileInfo(file).fileName().toLatin1() << tr(" (autogenerated by xyscan)") << Qt::endl;
+    out << tr("// xyscan Version ") << VERSION << Qt::endl;
     out << tr("// Date: ") << ctime(&now);
-    out << tr("// Scanned by: ") << QDir::home().dirName() << endl;
-    out << tr("// Source: ") << src << endl;
-    out << tr("// Comment: ") << userComment() << endl;
-    out << "//" << endl;
+    out << tr("// Scanned by: ") << QDir::home().dirName() << Qt::endl;
+    out << tr("// Source: ") << src << Qt::endl;
+    out << tr("// Comment: ") << userComment() << Qt::endl;
+    out << "//" << Qt::endl;
     
     //
     //   Include files
     //
-    out << "#include \"TROOT.h\"" << endl;
-    out << "#include \"TH1D.h\"" << endl;
-    out << "#include \"TCanvas.h\"" << endl;
-    out << "#include \"TStyle.h\"" << endl;
-    out << "#include \"TGraphAsymmErrors.h\"" << endl;
-    out << endl;
+    out << "#include \"TROOT.h\"" << Qt::endl;
+    out << "#include \"TH1D.h\"" << Qt::endl;
+    out << "#include \"TCanvas.h\"" << Qt::endl;
+    out << "#include \"TStyle.h\"" << Qt::endl;
+    out << "#include \"TGraphAsymmErrors.h\"" << Qt::endl;
+    out << Qt::endl;
     
     //
     //   Styles
     //
-    out << "void " << QFileInfo(file).baseName().toLatin1() << "()" << endl;
-    out << "{" << endl;
-    out << "    gROOT->SetStyle(\"Plain\");" << endl;
-    out << "    gStyle->SetOptFit(0);" << endl;
-    out << "    gStyle->SetOptStat(0);" << endl;
-    out << "    gStyle->SetEndErrorSize(5);" << endl;
-    out << endl;
+    out << "void " << QFileInfo(file).baseName().toLatin1() << "()" << Qt::endl;
+    out << "{" << Qt::endl;
+    out << "    gROOT->SetStyle(\"Plain\");" << Qt::endl;
+    out << "    gStyle->SetOptFit(0);" << Qt::endl;
+    out << "    gStyle->SetOptStat(0);" << Qt::endl;
+    out << "    gStyle->SetEndErrorSize(5);" << Qt::endl;
+    out << Qt::endl;
 
     //
     //   Canvas
     //
-    out << "    TCanvas *c1 = new TCanvas(\"c1\",\"xyscan Data Display\",720,540);" << endl;
-    out << "    c1->SetTickx(1);" << endl;
-    out << "    c1->SetTicky(1);" << endl;
-    out << "    c1->SetBorderSize(1);" << endl;
-    out << "    c1->SetFillColor(0);" << endl;
-    out << endl;
+    out << "    TCanvas *c1 = new TCanvas(\"c1\",\"xyscan Data Display\",720,540);" << Qt::endl;
+    out << "    c1->SetTickx(1);" << Qt::endl;
+    out << "    c1->SetTicky(1);" << Qt::endl;
+    out << "    c1->SetBorderSize(1);" << Qt::endl;
+    out << "    c1->SetFillColor(0);" << Qt::endl;
+    out << Qt::endl;
     
     //
     //  Cannot rely on markers being set at the actual plot
@@ -454,42 +464,42 @@ void xyscanDataTable::writeRootMacro(QFile& file, const QString& src, bool logx,
     ymin = min(ymin, markerCoord[2]);
     ymax = max(ymax, markerCoord[3]);
     
-    out << "    TH1D *histo = new TH1D(\"histo\",\"xyscan\", 100, " << xmin << ", " << xmax << ");" << endl;
-    out << "    histo->SetMinimum(" << ymin << ");" << endl;
-    out << "    histo->SetMaximum(" << ymax << ");" << endl;
-    out << "    histo->SetStats(false);" << endl;
-    out << "    histo->GetXaxis()->SetTitle(\"x\");" << endl;
-    out << "    histo->GetYaxis()->SetTitle(\"y\");" << endl;
-    out << "    histo->GetXaxis()->SetLabelFont(42);" << endl;
-    out << "    histo->GetYaxis()->SetLabelFont(42);" << endl;
-    out << "    histo->GetXaxis()->SetTitleFont(42);" << endl;
-    out << "    histo->GetYaxis()->SetTitleFont(42);" << endl;
-    out << "    gPad->SetLogy(" << (logy ? 1 : 0) << ");" << endl;
-    out << "    gPad->SetLogx(" << (logx ? 1 : 0) << ");" << endl;
+    out << "    TH1D *histo = new TH1D(\"histo\",\"xyscan\", 100, " << xmin << ", " << xmax << ");" << Qt::endl;
+    out << "    histo->SetMinimum(" << ymin << ");" << Qt::endl;
+    out << "    histo->SetMaximum(" << ymax << ");" << Qt::endl;
+    out << "    histo->SetStats(false);" << Qt::endl;
+    out << "    histo->GetXaxis()->SetTitle(\"x\");" << Qt::endl;
+    out << "    histo->GetYaxis()->SetTitle(\"y\");" << Qt::endl;
+    out << "    histo->GetXaxis()->SetLabelFont(42);" << Qt::endl;
+    out << "    histo->GetYaxis()->SetLabelFont(42);" << Qt::endl;
+    out << "    histo->GetXaxis()->SetTitleFont(42);" << Qt::endl;
+    out << "    histo->GetYaxis()->SetTitleFont(42);" << Qt::endl;
+    out << "    gPad->SetLogy(" << (logy ? 1 : 0) << ");" << Qt::endl;
+    out << "    gPad->SetLogx(" << (logx ? 1 : 0) << ");" << Qt::endl;
 
-    out << "    histo->Draw();" << endl;
-    out << endl;
+    out << "    histo->Draw();" << Qt::endl;
+    out << Qt::endl;
 
     //
     //    Setup arrays holding the data points and errors
     //
     int nrc = rowCount();
-    out << "    double x[" << nrc << "];" << endl;
-    out << "    double y[" << nrc << "];" << endl;
+    out << "    double x[" << nrc << "];" << Qt::endl;
+    out << "    double y[" << nrc << "];" << Qt::endl;
 
     char arrayName[64];
     int nGraphs = max(mXErrorScanCount, mYErrorScanCount);
     for (int i=0; i<nGraphs; i++) {
         sprintf(arrayName,"dxlow%d", i);
-        out << "    double " << arrayName << "[" << nrc << "];" << endl;
+        out << "    double " << arrayName << "[" << nrc << "];" << Qt::endl;
         sprintf(arrayName,"dxup%d", i);
-        out << "    double " << arrayName << "[" << nrc << "];" << endl;
+        out << "    double " << arrayName << "[" << nrc << "];" << Qt::endl;
     }
     for (int i=0; i<nGraphs; i++) {
         sprintf(arrayName,"dylow%d", i);
-        out << "    double " << arrayName << "[" << nrc << "];" << endl;
+        out << "    double " << arrayName << "[" << nrc << "];" << Qt::endl;
         sprintf(arrayName,"dyup%d", i);
-        out << "    double " << arrayName << "[" << nrc << "];" << endl;
+        out << "    double " << arrayName << "[" << nrc << "];" << Qt::endl;
     }
 
     //
@@ -497,7 +507,7 @@ void xyscanDataTable::writeRootMacro(QFile& file, const QString& src, bool logx,
     //    Note that dx_lower() etc are returning 0 if no point was scanned,
     //    exactly what we want here.
     //
-    out << "    int n = 0;" << endl;
+    out << "    int n = 0;" << Qt::endl;
     for (irow=0; irow < nrc; irow++) {
         out << "    x[n] = " << x(irow) << ";\t";
         out << "y[n] = " << y(irow) << ";\t";
@@ -513,9 +523,9 @@ void xyscanDataTable::writeRootMacro(QFile& file, const QString& src, bool logx,
             sprintf(arrayName,"dyup%d", i);
             out << arrayName << "[n] = " << dy_upper(irow, i) << ";\t";
         }
-        out << "n++;" << endl;
+        out << "n++;" << Qt::endl;
     }
-    out << endl;
+    out << Qt::endl;
 
     //
     //  Create graphs
@@ -528,22 +538,29 @@ void xyscanDataTable::writeRootMacro(QFile& file, const QString& src, bool logx,
     for (int i=0; i<nGraphs; i++) {
         sprintf(graphName,"xyscan%d", i);
         sprintf(arrayName,"dxlow%d, dxup%d, dylow%d, dyup%d", i, i, i, i);
-        out << "    TGraphAsymmErrors *" << graphName << " = new TGraphAsymmErrors(n, x, y, " << arrayName << ");" << endl;
-        out << "    " << graphName << "->SetMarkerColor(1);" << endl;
+        out << "    TGraphAsymmErrors *" << graphName << " = new TGraphAsymmErrors(n, x, y, " << arrayName << ");" << Qt::endl;
+        out << "    " << graphName << "->SetMarkerColor(1);" << Qt::endl;
         if (i==0) {
-            out << "    " << graphName << "->SetMarkerStyle(20);" << endl;
-            out << "    " << graphName << "->SetLineColor(1);" << endl;
-            out << "    " << graphName << "->Draw(\"PE same\");" << endl;
+            out << "    " << graphName << "->SetMarkerStyle(20);" << Qt::endl;
+            out << "    " << graphName << "->SetLineColor(1);" << Qt::endl;
+            out << "    " << graphName << "->Draw(\"PE same\");" << Qt::endl;
         }
         else {
             icolor++;
             if (icolor == 3) icolor++; // standard green in ROOT is too light
-            out << "    " << graphName << "->SetMarkerStyle(1);" << endl;
-            out << "    " << graphName << "->SetLineColor(" << icolor <<  ");" << endl;
-            out << "    " << graphName << "->Draw(\"PE same [] \");" << endl;
+            out << "    " << graphName << "->SetMarkerStyle(1);" << Qt::endl;
+            out << "    " << graphName << "->SetLineColor(" << icolor <<  ");" << Qt::endl;
+            out << "    " << graphName << "->Draw(\"PE same [] \");" << Qt::endl;
         }
     }
-    out << "}" << endl;
+    if (nGraphs == 0) {
+        sprintf(graphName,"xyscanGraph");
+        out << "    TGraph *" << graphName << " = new TGraph(n, x, y);" << Qt::endl;
+        out << "    " << graphName << "->SetMarkerStyle(20);" << Qt::endl;
+        out << "    " << graphName << "->SetLineColor(1);" << Qt::endl;
+        out << "    " << graphName << "->Draw(\"PE same\");" << Qt::endl;
+    }
+    out << "}" << Qt::endl;
     mDataSaved = true;
 }
 
@@ -553,11 +570,11 @@ void xyscanDataTable::writeTextFile(QFile& file, const QString& src)
     int irow, icol;
     time_t now = time(0);
 
-    out << tr("# xyscan Version ") << VERSION << endl;
+    out << tr("# xyscan Version ") << VERSION << Qt::endl;
     out << tr("# Date: ") << ctime(&now);
-    out << tr("# Scanned by: ") << QDir::home().dirName() << endl;
-    out << tr("# Source: ") << src << endl;
-    out << tr("# Comment: ") << userComment() << endl;
+    out << tr("# Scanned by: ") << QDir::home().dirName() << Qt::endl;
+    out << tr("# Source: ") << src << Qt::endl;
+    out << tr("# Comment: ") << userComment() << Qt::endl;
     out << tr("# Format: x, y");
     for (int i=0; i < mXErrorScanCount; i++) {
         out << ", -dx, +dx";
@@ -565,7 +582,7 @@ void xyscanDataTable::writeTextFile(QFile& file, const QString& src)
     for (int i=0; i < mYErrorScanCount; i++) {
         out << ", -dy, +dy";
     }
-    out << endl;
+    out << Qt::endl;
     for (irow=0; irow < rowCount(); irow++) {
         for (icol=0; icol < columnCount(); icol++) {
             if (item(irow, icol))
@@ -573,9 +590,19 @@ void xyscanDataTable::writeTextFile(QFile& file, const QString& src)
             else
                 out << "-" << "\t";
         }
-        out << endl;
+        out << Qt::endl;
     }
-    out << "# EoF" << endl;
+    out << "# EoF" << Qt::endl;
 
     mDataSaved = true;
 }
+
+double xyscanDataTable::value(int irow, int icol) const
+{
+    QLocale theLocal;
+    bool ok = false;
+    double result = 0;
+    if (item(irow, icol))  result = theLocal.toDouble(item(irow, icol)->text(), &ok);
+    if (!ok) result = 0;
+    return result;
+}
