@@ -39,8 +39,8 @@
 //   separatly.
 //
 #if defined(Q_OS_MAC)   // on macOS we use QtPDF
-#define  USE_POPPLER 1
-#undef  USE_QTPDF   // TMP ??????
+#define  USE_QTPDF 1
+#undef  USE_POPPLER
 #elif defined(Q_OS_WIN) // on Windows we have no good solution (yet)
 #undef  USE_QTPDF
 #undef  USE_POPPLER
@@ -88,6 +88,7 @@ xyscanWindow::xyscanWindow()
     connectMenuActions();
     connectToolBoxActions();
     createSettingMarkerActions();
+    connectDockingWidgets();
 
     //
     //  Create sound "click" sound effect
@@ -109,9 +110,9 @@ xyscanWindow::xyscanWindow()
     //  Shortcuts to Coordinates and Axis Settings pages
     //  that are the most used ones.
     //
-    QShortcut *shortcut1 = new QShortcut(Qt::CTRL + Qt::Key_C, mToolBox);
+    QShortcut *shortcut1 = new QShortcut(Qt::CTRL | Qt::Key_C, mToolBox);
     connect(shortcut1, &QShortcut::activated, [=](){mToolBox->setCurrentIndex(3);});
-    QShortcut *shortcut2 = new QShortcut(Qt::CTRL + Qt::Key_A, mToolBox);
+    QShortcut *shortcut2 = new QShortcut(Qt::CTRL | Qt::Key_A, mToolBox);
     connect(shortcut2, &QShortcut::activated, [=](){mToolBox->setCurrentIndex(1);});
 
     //
@@ -240,7 +241,7 @@ void xyscanWindow::checkForUpdates()
 bool xyscanWindow::checkForUnsavedData()
 {
     //
-    //  Allows he user to save the data before continuing.
+    //  Allows the user to save the data before continuing.
     //  Method returns true when data was saved or explicitly
     //  discarded or false when Cancel was pressed.
     //
@@ -354,10 +355,10 @@ void xyscanWindow::connectToolBoxActions()
     //  Ctrl+N.
     //
     connect(mMeasureUserScaleField, &QLineEdit::returnPressed, [=](){if (QGuiApplication::queryKeyboardModifiers()== Qt::ShiftModifier) this->normalizeUserScale();});
-    QShortcut *shortcutN = new QShortcut(Qt::CTRL + Qt::Key_N, mMeasureItem);
+    QShortcut *shortcutN = new QShortcut(Qt::CTRL | Qt::Key_N, mMeasureItem);
     connect(shortcutN, &QShortcut::activated, [=](){this->normalizeUserScale();});
 
-    QShortcut *shortcutM = new QShortcut(Qt::CTRL + Qt::Key_M, mMeasureItem);
+    QShortcut *shortcutM = new QShortcut(Qt::CTRL | Qt::Key_M, mMeasureItem);
     connect(shortcutM, &QShortcut::activated, [=](){this->enableMeasuring(!mMeasuringIsOn);});
 }
 
@@ -368,10 +369,10 @@ void xyscanWindow::createSettingMarkerActions()
     connect(mSetLowerYButton, &QPushButton::clicked, [=](){this->setPlotCoordinateValueForMarker(mYLower);});
     connect(mSetUpperYButton, &QPushButton::clicked, [=](){this->setPlotCoordinateValueForMarker(mYUpper);});
     
-    QShortcut *shortcut1 = new QShortcut(Qt::CTRL + Qt::Key_1, mSetLowerXButton);
-    QShortcut *shortcut2 = new QShortcut(Qt::CTRL + Qt::Key_2, mSetUpperXButton);
-    QShortcut *shortcut3 = new QShortcut(Qt::CTRL + Qt::Key_3, mSetLowerYButton);
-    QShortcut *shortcut4 = new QShortcut(Qt::CTRL + Qt::Key_4, mSetUpperYButton);
+    QShortcut *shortcut1 = new QShortcut(Qt::CTRL | Qt::Key_1, mSetLowerXButton);
+    QShortcut *shortcut2 = new QShortcut(Qt::CTRL | Qt::Key_2, mSetUpperXButton);
+    QShortcut *shortcut3 = new QShortcut(Qt::CTRL | Qt::Key_3, mSetLowerYButton);
+    QShortcut *shortcut4 = new QShortcut(Qt::CTRL | Qt::Key_4, mSetUpperYButton);
 
     connect(shortcut1, &QShortcut::activated, [=](){this->setPlotCoordinateValueForMarker(mXLower);});
     connect(shortcut2, &QShortcut::activated, [=](){this->setPlotCoordinateValueForMarker(mXUpper);});
@@ -1511,9 +1512,12 @@ void xyscanWindow::rotateImage(double d)
         //  Right now, have no solution to get the proper
         //  coordinate in the pixmap after rotating. Disable for now.
         //
-        if (mHistogramDock->isVisible()) mHistogramDock->setHidden(true);
-        mHistogramDock->toggleViewAction()->setDisabled(true);
-
+        if (mImageScale != 1 || mImageAngle != 0) {
+            if (mHistogramDock->isVisible()) mHistogramDock->setHidden(true);
+            mHistogramDock->toggleViewAction()->setDisabled(true);
+        }
+        else
+            mHistogramDock->toggleViewAction()->setDisabled(false);
         update();
     }
 }
@@ -1577,8 +1581,12 @@ void xyscanWindow::scaleImage(double z)
         //  Right now, have no solution to get the proper
         //  coordinate in the pixmap after scaling. Disable for now.
         //
-        if (mHistogramDock->isVisible()) mHistogramDock->setHidden(true);
-        mHistogramDock->toggleViewAction()->setDisabled(true);
+        if (mImageScale != 1 || mImageAngle != 0) {
+            if (mHistogramDock->isVisible()) mHistogramDock->setHidden(true);
+            mHistogramDock->toggleViewAction()->setDisabled(true);
+        }
+        else
+            mHistogramDock->toggleViewAction()->setDisabled(false);
 
         update();
     }
